import Async, { CustomSignalClass, Signal } from "typescene-async";
import { ActivityResourcePath } from "./ActivityPath";
import { ActivityTransition } from "./ActivityStack";
/** Represents a UI activity */
export declare class Activity extends Async.ObservableObject {
    /** Get an instance of this activity class, can be overridden to alter behavior when e.g. pushing a parent activity by class; by default simply constructs the instance without parameters */
    static getInstance(): Activity;
    /** Create an activity with given display title */
    constructor(title?: string);
    /** Display title of this activity (observable) */
    title: string | undefined;
    /** Object that represents the internal state of the activity; this is a sealed observable object, never null or undefined, and setting this to a plain object ({...}) will create an ObservableObject instance (in practice, initialize all properties as activity.state = { x: ... } first to make properties observable); it is recommended to keep properties on the state object instead of on the Activity instance itself, and provide public access through the derived Activity's methods (e.g. to view instances) */
    protected state: {};
    /** Object that contains options for this activity (observable, not null or undefined) */
    options: Activity.ActivityOptions;
    /** Signal that is emitted before this activity is pushed to foreground, if any handler throws an error then the activity is not activated */
    readonly Activating: CustomSignalClass<ActivityTransition, Signal<ActivityTransition>>;
    /** Signal that is emitted after this activity is pushed to foreground */
    readonly Activated: CustomSignalClass<ActivityTransition, Signal<ActivityTransition>>;
    /** Signal that is emitted before another activity is pushed to foreground, if any handler throws an error then the activity is not suspended */
    readonly Suspending: CustomSignalClass<ActivityTransition, Signal<ActivityTransition>>;
    /** Signal that is emitted when this activity is no longer in foreground */
    readonly Suspended: CustomSignalClass<ActivityTransition, Signal<ActivityTransition>>;
    /** Signal that is emitted before this activity is put back in foreground, if any handler throws an error then the activity is not suspended */
    readonly Resuming: CustomSignalClass<ActivityTransition, Signal<ActivityTransition>>;
    /** Signal that is emitted when this activity is back in foreground */
    readonly Resumed: CustomSignalClass<ActivityTransition, Signal<ActivityTransition>>;
    /** Signal that is emitted after this activity is removed from history */
    readonly Deactivated: CustomSignalClass<ActivityTransition, Signal<ActivityTransition>>;
}
export declare namespace Activity {
    /** Options for an activity that determine its behavior in the activity stack */
    interface ActivityOptions {
        /** The parent activity, if any; the activity stack will ensure the given activity (or an instance of given class) exists in its history stack when activating this activity, or insert it before */
        parentActivity?: Activity | typeof Activity;
        /** Set to true to require this activity to be in the foreground, or move up to it, when used as a parent activity; i.e. when starting or resuming a child activity, all activities on top of the hub activity are suspended and replaced with the child activity; can be used to prevent buildup of activities on the history stack and therefore memory leaks */
        isHubActivity?: boolean;
        /** Set to true to allow only one copy of an activity on the stack; i.e. when pushing an instance that is already on the stack, the stack is transitioned up to the existing instance instead; use with a singleton activity to implement e.g. an application home activity */
        isRootActivity?: boolean;
    }
}
/** Represents an activity pertaining to a resource (URL path) */
export declare class ResourceActivity extends Activity {
    /** Create an activity with given resource path */
    constructor(title?: string, path?: ActivityResourcePath | string);
    /** The path (URL/hash segment) that is associated with this activity */
    resourcePath: ActivityResourcePath | string | undefined;
}
export default Activity;
