import { CustomSignalClass, Signal } from "typescene-async";
import Activity from "./Activity";
import { Component } from "../";
/** Represents a stack of activated activities (like browser history) */
export declare class ActivityStack {
    /** Add an activity to the foreground asynchronously, does nothing if given activity was already in the foreground; returns Promise that resolves to the completed transition */
    pushAsync(activity: Activity): PromiseLike<ActivityTransition>;
    /** Replace the current activity asynchronously (throws error if none), or remove current activity if given activity was already directly below current activity in the activity stack; returns Promise that resolves to the completed transition */
    replaceAsync(activity: Activity): PromiseLike<ActivityTransition>;
    /** Remove the current foreground activity (go back) asynchronously, only given activity if any (otherwise any activity that is currently in the foreground), returns Promise that resolves to the completed transition, if any */
    popAsync(activity?: Activity): PromiseLike<ActivityTransition | null>;
    /** Remove foreground activities until given activity or activity of given type is in the foreground; returns Promise that resolves to activity, or null if there was no matching activity on the stack */
    upAsync(activityOrClass: Activity | typeof Activity): PromiseLike<Activity | null>;
    /** Reload state using given history ID, if possible (i.e. not yet deactivated relevant activities in the meantime); returns a promise that resolves when the state has been reached */
    restoreHistoryStateAsync(historyID: string): PromiseLike<void>;
    /** Get an ID that represents the current state, for use with reset(...) */
    getHistoryState(): string;
    /** Get the activity closest to the foreground of the given type, if any (excluding foreground activity itself, and before given activity in second parameter, if any) */
    getParent(ActivityClass: typeof Activity, before?: Activity): Activity | undefined;
    /** Get view component instance for given named outlet, from the view activity closest to the foreground that has given outlet mapped using `.mapView(...)` or the `mapToActivity(...)` decorator on a view Component class */
    getViewByName(outlet: string): Component | undefined;
    /** Returns true if the stack contains given activity */
    contains(activity: Activity): boolean;
    /** The current foreground activity (top of stack, if any; observable) */
    readonly top: Activity | undefined;
    /** The number of activities on the stack (observable) */
    readonly length: number;
    /** The title of the topmost activity that has a title defined (observable) */
    readonly title: string | undefined;
    /** Signal that is emitted when a transition occurs (after Activating/Resuming/Suspending but before Activated/Resumed) */
    readonly Transition: CustomSignalClass<ActivityTransition, Signal<ActivityTransition>>;
    /** Helper method to find parent activity/ies for given activity; returns an array of activities to be pushed (including activity itself) */
    private _findParents(activity, excludeTop?);
    /** Helper method to move up to existing root activity, or hub parent activity if needed */
    private _upToHubOrRoot(activity);
    /** Helper method for sending signals and creating promise */
    private _processTransition(to, operation);
    private _topIndex;
    private _stack;
    private _ids;
    private _transitionP;
}
/** Represents a transition from one foreground activity to another */
export interface ActivityTransition {
    /** The activity stack that is transitioning */
    activityStack: ActivityStack;
    /** The activity that is currently in the foreground, if any */
    from: Activity;
    /** The new foreground activity, if any */
    to: Activity;
    /** The stack operation being performed: push, replace, or pop */
    operation: ActivityTransition.Operation;
    /** The previous transition that is part of the same operation (pop/push/pop/up) */
    previous: ActivityTransition;
}
export declare namespace ActivityTransition {
    /** Operation type that triggered a transition */
    enum Operation {
        /** Push operation */
        Push = 0,
        /** Replace operation */
        Replace = 1,
        /** Pop operation */
        Pop = 2,
    }
}
export default ActivityStack;
