import { CustomSignalClass, Signal } from "typescene-async";
import { Component } from "../";
import Activity, { ResourceActivity } from "./Activity";
import { ActivityTransition } from "./ActivityStack";
import { ActivityResourcePath } from "./ActivityPath";
/** Represents an activity that corresponds to a (component on) screen */
export declare class ViewActivity extends ResourceActivity {
    /** Create a view activity with optional title and view component instance, parent reference, and resource path; to add a view component dynamically, use the `mapToActivity(...)` decorator on a view Component class or call the `.mapView(...)` method on this activity */
    constructor(title?: string, path?: ActivityResourcePath | string, view?: Component, parent?: Activity | typeof Activity);
    /** The main view component that is currently displayed, if any; this is a readonly field, to link a view to an activity use the `mapToActivity(...)` decorator on a view Component class or call the `.mapView(...)` method on a view activity */
    readonly view: Component | undefined;
    /** Signal that is emitted before this activity's view is displayed in the foreground (i.e. NOT if displayed as parent view) */
    readonly Displaying: CustomSignalClass<ViewActivityTransition, Signal<ViewActivityTransition>>;
    /** Signal that is emitted after this activity is displayed in the foreground (i.e. NOT if displayed as parent view) */
    readonly Displayed: CustomSignalClass<ViewActivityTransition, Signal<ViewActivityTransition>>;
    /** Register given function to return view component instance for given outlet name, or main view if none given; existing getter for same outlet is replaced, if any; getter must always return a component instance, which may or may not be the same instance returned by previous calls */
    mapView(f: () => Component, outlet?: string): void;
    /** Get an instance of the view component for given named outlet on this activity, or main view if none given, by calling the mapped view getter that has been registered through the `.mapView(...)` method (within unobserved function wrapper to avoid observable side effects) */
    getViewByName(outlet?: string): Component | undefined;
    /** Display the view component */
    private _displayMainView(resumed, t, child?);
    /** Remove the view component from the screen */
    private _removeView(t);
    /** View getter(s), registered through mapView method; main component getter stored in empty string property */
    private _viewGetters;
    /** View currently displayed */
    private _view;
}
/** Represents an activity that corresponds to a component on screen, created only once and re-used if required (overrides .getInstance static method); abstract class, needs to be overridden */
export declare abstract class SingletonViewActivity extends ViewActivity {
    /** Get the instance of this activity class, constructs the instance without parameters once and returns this instance every time */
    static getInstance(): SingletonViewActivity;
    /** Create a new activity; can be used only once */
    constructor(title?: string, path?: ActivityResourcePath | string, view?: Component, parent?: Activity | typeof Activity);
    private static _instance;
}
/** Represents a singleton view activity that is both a root activity (i.e. may exist only once on the activity stack, starting/replacing the activity again transitions the stack up to the existing activity instead), and a hub activity (i.e. starting any activity that specifies this activity as its parent activity, transitions the stack up to the hub activity first, suspending all other activities that used to be in the foreground); abstract class, needs to be overridden */
export declare abstract class RootViewActivity extends SingletonViewActivity {
    constructor(title?: string, path?: ActivityResourcePath | string, view?: Component);
}
/** Represents a transition from one (view) activity to a view activity that is to be displayed, encapsulates activity transition */
export interface ViewActivityTransition {
    /** The activity transition itself */
    transition: ActivityTransition;
    /** The main view, if any, displayed as a result of this transition */
    view: Component | undefined;
    /** True if displaying this view for the first time (i.e. the activity has just been added, and not resumed or added again) */
    firstDisplay: boolean;
    /** True if adding this view to the browser history (i.e. not resuming from history) */
    forward: boolean;
    /** Number of transitions in the current chain (for checking if e.g. a dropActivity(...) call crossed multiple activities to get to this view) */
    hops: number;
}
/** Interface definition for a view class that has a constructor taking a single Activity argument */
export interface ActivityViewClass<ActivityT extends Activity> {
    new (activity: ActivityT): Component;
}
/** Class decorator, maps the decorated view Component class (with a constructor that has a single matching activity argument) to a ViewActivity class as given named outlet, or as the main view if none specified; does not map derived activity and/or view classes; views are dereferenced within given timeout (in ms, defaults to 100, set to 0 to disable) when no longer displayed on screen */
export declare function mapToActivity<ActivityT extends ViewActivity>(activityClass: {
    new (...args: any[]): ActivityT;
}, outlet?: string, dereferenceTimeout?: number): (target: ActivityViewClass<ActivityT>) => void;
export default ViewActivity;
