import Async, { CustomSignalClass, Signal } from "typescene-async";
import { Activity, ResourceActivity, ActivityTransition, ActivityStack, ActivityResourcePath, PathInitializerPart } from "./Activity";
import { Component } from "./Components";
/** Encapsulates the application as a singleton object */
export declare class Application extends Async.ObservableObject {
    /** Map resource activity on its path (using .resourcePath) */
    static mapActivity(activity: ResourceActivity): void;
    /** Map given path (#/... or /...) to given activity (i.e. instance or instance of class through static getInstance method) */
    static mapActivity(path: PathInitializerPart | PathInitializerPart[], activity?: Activity | typeof Activity): void;
    /** Map given path (#/... or /...) to given callback; if callback has no parameters, an exact path match is required; otherwise callback is called with path remainder split on / characters plus glob (last parameter); callback should return an activity or array of activities (or a promise that resolves to either), returned ResourceActivity instances without a resource path will get the requested path set as their resource path automatically */
    static mapResource(path: PathInitializerPart | PathInitializerPart[], callback: (...params: string[]) => Activity | Activity[] | PromiseLike<Activity | Activity[]>): Async.SignalConnection;
    /** Disable synchronization of activity stack with browser history */
    static disableBrowserHistory(): void;
    /** The current (and only) Application instance, when created */
    static current: Application;
    /** Promise that resolves to the application instance after it has been created */
    static ready: Async.Promise<Application>;
    private static _resolve_ready;
    /** Create the application instance (either from a derived class or from the Application class itself); can be called only once */
    constructor(title?: string);
    /** The application name to be displayed in the title bar (observed), overridden by the title of the current activity/ies, if any */
    title: string;
    /** Signal that is emitted when no activity is found for a location */
    readonly PageNotFound: CustomSignalClass<string, Signal<string>>;
    /** Signal that is emitted when all activities have been dropped and none have been started */
    readonly NoActivity: CustomSignalClass<void, Signal<void>>;
    /** Activity manager instance */
    activities: ActivityStack;
    /** Map resource activity on its path (using .resourcePath); returns this; also available as a static method */
    mapActivity(activity: ResourceActivity): void;
    /** Map given path (#/... or /...) to given activity (i.e. instance or instance of class through static getInstance method); returns this; also available as a static method */
    mapActivity(path: PathInitializerPart | PathInitializerPart[], activity?: Activity | typeof Activity): void;
    /** Map given path (#/... or /...) to given callback; if callback has no parameters, an exact path match is required; otherwise callback is called with path remainder split on / characters except last parameter (glob); callback should return an activity or array of activities (or a promise that resolves to either), returned ResourceActivity instances without a resource path will get the requested path set as their resource path automatically; returns this; also available as a static method */
    mapResource(path: PathInitializerPart | PathInitializerPart[], callback: (...params: string[]) => Activity | Activity[] | PromiseLike<Activity | Activity[]>): this;
    /** Get view component instance for given named outlet, from the view activity closest to the foreground that has given outlet mapped using `.mapView(...)` or the `mapToActivity(...)` decorator on a view Component class; can be used as an observable getter to obtain an observable value that changes when activities are started or suspended */
    getViewByName(outlet: string): Component | undefined;
    /** Start given activity as foreground activity (push/replace, same as calling .activities.push/replace(...) directly), or start activity using given resource path or string (#/... or /..., or relative path as #./..., #../..., ./... or ../...); set navigateIfNotFound to true to set window.location.href to the new location if no matching activity is found (for external URLs), otherwise invokes onPageNotFound handler; note that activities transition asynchronously and any updates may not be reflected until several milliseconds after calling this method */
    startActivity(activityOrPath: Activity | typeof Activity | ActivityResourcePath | string, replace?: boolean, setHrefIfNotFound?: boolean): PromiseLike<ActivityTransition[]>;
    /** Go back in activity history; either to previous activity or to activity of given class; returns promise that resolves to the new foreground activity, or null if activity was not found; the NoActivity signal is emitted if all activities have been dropped and there is no current activity anymore */
    dropActivity(ActivityClass?: typeof Activity): PromiseLike<Activity | null>;
    private _listen();
    private _startLocationActivity(replace?);
    private _updateHistoryState(op?);
}
/** Class decorator, maps the decorated Activity class to given path (using `Application.mapActivity` method) */
export declare function mapToPath(path: PathInitializerPart | PathInitializerPart[]): (target: typeof Activity) => void;
/** Class decorator, maps the decorated Activity class to simple sub resources on given path (using `Application.mapResource` method, use that method directly for more options); the activity class *must* have a constructor that takes a single string parameter, i.e. the resource path glob */
export declare function mapToResource(path: PathInitializerPart | PathInitializerPart[]): (target: new (glob: string) => Activity) => void;
export default Application;
