import Async from "typescene-async";
import { Label } from "../";
/** Shared factory implementation for a piece of text as a string and/or as (an observable value representing) a Label instance (or sub class); can be used in Component factory specs using UI.tl(...) or UI.tl`...` */
export declare class TextLabelFactory {
    /** Create the text label factory using the given string, stringable (object with .toString method) or an observable value, and a Label base class that is used if not overridden by string content prefix */
    constructor(text: {
        toString;
    } | Async.ObservableValue<{
        toString;
    }>, baseClass?: typeof Label);
    /** Get the encapsulated text as a string; adds a dependency on the observable value if called from an obserable value getter; trims {...} prefix if present */
    toString(): string;
    /** Get the encapsulated text wrapped in a new Label (or derived) instance, or an observable value representing the instance; uses {...} prefix to control component type and its properties */
    getComponent(): Label | Async.ObservableValue<Label>;
    private _createComponent(text, prev?);
    private _baseClass;
    private _value;
}
export default TextLabelFactory;
/**
 * Tagged template string function that returns a text label factory instance to be used with component factory content lists; functions in the expanded values (i.e. result of ${...}) are used as getters for nested observable string values; prefixes can be used to change the type of component factory created and its styles and properties:
 * * `{p}` for Paragraph,
 * * `{h1-6}` for Heading,
 * * `{w}` for WideLabel,
 * * `{b|i|u}` for text styles,
 * * `{left|center|right}` for alignment,
 * * `{#...}` for text colors,
 * * `{nnn}` for font weights,
 * * `{icon:...}` for icons,
 * * `{icon:... property-name=...}` for icon style properties,
 * * `{property-name=...}` or `{propertyName=...}` for individual style properties,
 * * `{...%|...em|...rem| etc.}` for font sizes, and
 * * `{l:...%|...rem| etc.}` for Label with given width;
 *
 * Prefixes can be combined using the pipe | symbol and may also be the result of an expanded value (i.e. first ${...} part)
 */
export declare function tl(strings: TemplateStringsArray, ...values: any[]): TextLabelFactory;
/** Simplified version of tagged template helper .tl`...` with a single piece of text, possibly with a base Label class that is used if given string does not override the class using e.g. {p} or {h1} */
export declare function tl(text: string | Async.ObservableValue<string>, baseClass?: typeof Label): TextLabelFactory;
/** Simplified version of tagged template helper .tl`...` observing a single piece of text, possibly with a base Label class that is used if given string does not override the class using e.g. {p} or {h1} */
export declare function tl(getter: () => (string | Async.ObservableValue<string>), baseClass?: typeof Label): TextLabelFactory;
